package eu.mhsl.craftattack.spawn.api;

import com.google.gson.Gson;
import eu.mhsl.craftattack.spawn.Main;
import eu.mhsl.craftattack.spawn.appliance.Appliance;
import org.bukkit.configuration.ConfigurationSection;
import spark.Request;
import spark.Spark;

import java.util.function.Function;

public class HttpServer {
    private final ConfigurationSection apiConf = Main.instance().getConfig().getConfigurationSection("api");
    protected final Gson gson = new Gson();

    public static Object nothing = "";

    public HttpServer() {
        Spark.port(8080);

        Spark.get("/ping", (request, response) -> System.currentTimeMillis());

        Main.instance().getAppliances().forEach(appliance -> appliance.httpApi(new ApiBuilder(appliance)));
    }

    public class ApiBuilder {
        @FunctionalInterface
        public interface RequestProvider<TParsed, TOriginal, TResponse> {
            TResponse apply(TParsed parsed, TOriginal original);
        }

        private final String applianceName;
        private ApiBuilder(Appliance appliance) {
            this.applianceName = appliance.getClass().getSimpleName().toLowerCase();
        }

        public void get(String path, Function<Request, Object> onCall) {
            Spark.get(this.buildRoute(path), (req, resp) -> HttpServer.this.gson.toJson(onCall.apply(req)));
        }

        public <TRequest> void post(String path, Class<TRequest> clazz, RequestProvider<TRequest, Request, Object> onCall) {
            Spark.post(this.buildRoute(path), (req, resp) -> {
                TRequest parsed = new Gson().fromJson(req.body(), clazz);
                Object response = onCall.apply(parsed, req);
                return HttpServer.this.gson.toJson(response);
            });
        }

        public String buildRoute(String path) {
            return String.format("/api/%s/%s", this.applianceName, path);
        }
    }
}
